from unittest import SkipTest
from odoo import Command
from odoo.tests import tagged
from odoo.addons.sale_purchase.tests.common import TestCommonSalePurchaseNoChart


@tagged('-at_install', 'post_install')
class TestServicePurchase(TestCommonSalePurchaseNoChart):

    @classmethod
    def setUpClass(cls, chart_template_ref=None):
        try:
            super().setUpClass(chart_template_ref=chart_template_ref)
        except SkipTest:
            # we didn't found the default CoA, we are probably testing w/
            # openerp_enterprise, so fallback to l10n_be
            super().setUpClass(chart_template_ref="l10n_be.l10nbe_chart_template")

        # Set country on customer as it's mandatory
        cls.partner_a.country_id = cls.env.ref("base.be")

        # Set all service product invoicing on 'order'
        (cls.service_purchase_1 | cls.service_purchase_2).invoice_policy = 'order'

        # create a generic Sale Order with 2 classical products and a purchase service
        SaleOrder = cls.env['sale.order'].with_context(tracking_disable=True)
        cls.sale_order_1 = SaleOrder.create({
            'partner_id': cls.partner_a.id,
            'order_line': [
                Command.create({
                    'product_id': cls.company_data['product_service_delivery'].id,
                    'product_uom_qty': 1,
                }),
                Command.create({
                    'product_id': cls.company_data['product_order_no'].id,
                    'product_uom_qty': 2,
                }),
                Command.create({
                    'product_id': cls.service_purchase_1.id,
                    'product_uom_qty': 4,
                })
            ]
        })
        cls.sol1_service_purchase_1 = cls.sale_order_1.order_line.filtered(
            lambda l: l.product_id == cls.service_purchase_1
        )
        cls.sol1_service_purchase_1.update_supplier()

        cls.sale_order_2 = SaleOrder.create({
            'partner_id': cls.partner_a.id,
            'order_line': [
                Command.create({
                    'product_id': cls.company_data['product_delivery_no'].id,
                    'product_uom_qty': 5,
                }),
                Command.create({
                    'product_id': cls.company_data['product_service_order'].id,
                    'product_uom_qty': 6,
                }),
                Command.create({
                    'product_id': cls.service_purchase_2.id,
                    'product_uom_qty': 7,
                })
            ]
        })
        cls.sol2_service_purchase_2 = cls.sale_order_2.order_line.filtered(
            lambda l: l.product_id == cls.service_purchase_2
        )
        cls.sol2_service_purchase_2.update_supplier()

    def test_invoice_payment_create_purchase(self):
        """ Paying a invoice with a service that should create a PO """

        self.sale_order_1.action_confirm()
        self.sale_order_2.action_confirm()
        purchase_order = self.env['purchase.order'].search([
            ('partner_id', '=', self.supplierinfo1.partner_id.id),
            ('state', '=', 'draft')
        ])
        self.assertFalse(purchase_order, "There should be no service PO created upon SO confirmation")

        so_1_invoice = self.sale_order_1._create_invoices()._post()
        so_2_invoice = self.sale_order_2._create_invoices()._post()
        purchase_order = self.env['purchase.order'].search([
            ('partner_id', '=', self.supplierinfo1.partner_id.id),
            ('state', '=', 'draft')
        ])
        self.assertFalse(purchase_order, "There should be no service PO created until invoices are paid")

        self.env['account.payment.register']\
            .with_context(active_model='account.move', active_ids=so_1_invoice.ids)\
            .create({
                'journal_id': self.company_data['default_journal_bank'].id,
                'payment_method_line_id': self.inbound_payment_method_line.id,
            })\
            ._create_payments()
        self.assertEqual(so_1_invoice.payment_state, 'in_payment')

        self.env['account.payment.register']\
            .with_context(active_model='account.move', active_ids=so_2_invoice.ids)\
            .create({
                'journal_id': self.company_data['default_journal_bank'].id,
                'payment_method_line_id': self.inbound_payment_method_line.id,
            })\
            ._create_payments()
        self.assertEqual(so_2_invoice.payment_state, 'in_payment')

        purchase_order = self.env['purchase.order'].sudo().search([
            ('partner_id', '=', self.supplierinfo1.partner_id.id),
            ('state', '=', 'draft'),
        ])
        self.assertTrue(purchase_order, "There should be service PO created after invoice payment")
        self.assertEqual(len(purchase_order), 1, "Only one PO should have been created, from the 2 invoices")
        self.assertEqual(purchase_order.company_id.id, 1, "Company should always be created in Odoo S.A.")
        self.assertEqual(purchase_order.order_line[0].product_id, self.service_purchase_1)
        self.assertEqual(purchase_order.order_line[1].product_id, self.service_purchase_2)

        refund_result = self.env['account.move.reversal']\
            .create({
                'journal_id': self.company_data['default_journal_sale'].id,
                'move_ids': [Command.link(so_1_invoice.id)],
            })\
            .refund_moves()
        refund_move_id = self.env[refund_result['res_model']].browse(refund_result['res_id'])
        refund_move_id._post()
        self.assertEqual(len(purchase_order.order_line), 2, "A refund should not create new PO lines until it is paid")

        self.env['account.payment.register']\
            .with_context(active_model='account.move', active_ids=refund_move_id.ids)\
            .create({
                'journal_id': self.company_data['default_journal_bank'].id,
                'payment_method_line_id': self.inbound_payment_method_line.id,
            })\
            ._create_payments()
        self.assertEqual(len(purchase_order.order_line), 3, "A refund should create new PO lines when it is paid")
        self.assertEqual(purchase_order.order_line[2].product_id, self.service_purchase_1)
        self.assertEqual(purchase_order.order_line[2].product_qty, -4, "The quantity on the new line should be negative")
